using System;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Net;
using System.Net.Sockets;

class TcpEchoClientGUI {

  public static void Main(String[] args) {

    if (args.Length > 2) { // Test for correct # of args
      throw new ArgumentException("Parameters: [<Server>] [<Port>]");
    }

    // Use server argument if supplied, otherwise default to localhost
    String server = (args.Length > 1) ? args[0] : "127.0.0.1"; 

    // Use port argument if supplied, otherwise default to 7
    int servPort = (args.Length == 2) ? Int32.Parse(args[1]) : 7;

    TcpEchoClient client = new TcpEchoClient(server, servPort);

    ClientGUI gui = new ClientGUI(client);

    Application.Run(gui);

  }

}

class TcpEchoClient {
  private String server ="";
  private int servPort = 0;

  public TcpEchoClient(String server, int servPort) {
    this.server = server;
    this.servPort = servPort;
  }

  public String getServer() { return server; }
  public int getPort() { return servPort; }

  public void sendEchoText(String text, Label statusLabel, TextBox receiveText) {
    TcpClient client = null;
    NetworkStream netStream = null;

    try {
      // Create socket that is connected to server on specified port
      client = new TcpClient(server, servPort);

      statusLabel.Text="Connected to server... sending echo text.";
    
      netStream = client.GetStream();

      byte[] byteBuffer = Encoding.ASCII.GetBytes(text);

      // Send the encoded string to the server
      netStream.Write(byteBuffer, 0, byteBuffer.Length);

      statusLabel.Text="Sent " + byteBuffer.Length + " bytes to server...";

      int totalBytesRcvd = 0;   // Total bytes received so far
      int bytesRcvd = 0;        // Bytes received in last read

      // Receive the same string back from the server
      while (totalBytesRcvd < byteBuffer.Length) {
        if ((bytesRcvd = netStream.Read(byteBuffer, totalBytesRcvd, 
                byteBuffer.Length - totalBytesRcvd)) == 0) {
          statusLabel.Text="Connection closed prematurely.";
          break;
        }
        totalBytesRcvd += bytesRcvd;
      }

      statusLabel.Text = "Received " + totalBytesRcvd + " bytes from server.";
      receiveText.AppendText(Encoding.ASCII.GetString(byteBuffer, 0, totalBytesRcvd) + "\r\n");

    } catch (Exception e) {
       statusLabel.Text = e.Message;
    } finally {
      if (netStream != null) netStream.Close();
      if (client != null) client.Close();
    }
  }
}

class ClientGUI:Form {

  private TextBox receiveText;
  private TextBox sendText;
  private Label statusLabel;
  private Label serverLabel;

  private TcpEchoClient client = null; // Class with socket echo client functionality

  private static readonly int LINE_RETURN_SIZE = 2; // No. of chars in a line return (\r\n)

  public ClientGUI(TcpEchoClient client) {

    this.client = client;

    // Create status label:
    serverLabel = new Label();
    serverLabel.Size = new Size(496, 20);
    serverLabel.Location = new Point(6, 18);
    serverLabel.Dock = DockStyle.Top;
    serverLabel.Text = "Echo Server: " + client.getServer() + ":" + client.getPort();

    // Create a text box to receive echo data 
    receiveText = new TextBox();

    receiveText.Size = new Size(500, 170);
    receiveText.Location = new Point(0, 0);
    receiveText.Dock = DockStyle.Top;
    receiveText.Multiline = true;
    receiveText.ScrollBars = ScrollBars.Both;
    receiveText.ReadOnly = true;
    receiveText.WordWrap = false;
    receiveText.Text = "";

    // Create a text box to send echo data
    sendText = new TextBox();

    sendText.Size = new Size(500, 40);
    sendText.Location = new Point(0, 172);
    sendText.Dock = DockStyle.Fill;
    sendText.Multiline = true;
    sendText.ScrollBars = ScrollBars.None;
    sendText.WordWrap = false;
    sendText.Text = "";
    sendText.TextChanged += new EventHandler(HandleTextChange);
    sendText.Focus();

    // Create status label:
    statusLabel = new Label();
    statusLabel.Size = new Size(500, 20);
    statusLabel.Location = new Point(0, 158);
    statusLabel.Dock = DockStyle.Bottom;
    statusLabel.Text = "Not connected.";

    Panel pan = new Panel();
    pan.Anchor = (AnchorStyles.Top|AnchorStyles.Bottom|
                    AnchorStyles.Left|AnchorStyles.Right);
    pan.Size = new Size(500, 218);
    pan.Location = new Point(2, 22);

    pan.Controls.AddRange(new Control[] {sendText, receiveText});

    AutoScaleBaseSize = new Size(5,13);
    ClientSize = new Size(504,266);
    Controls.AddRange(new Control[] {serverLabel, pan, statusLabel});
    Text = "TcpEchoClientGUI";
  }

  // Handle text change notifications and send echo string

  private void HandleTextChange(Object sender, EventArgs e){
     String strText = (sender as TextBox).Text; 

     if (strText.Length == 0) return;

     String strLastChar = strText.Substring(strText.Length-1, 1);

     // Check if user hit return (\n):
     if (strLastChar.Equals("\n")) {

        if (strText.Length - LINE_RETURN_SIZE == 0) {
           // nothing to send
           sendText.Text = "";
           statusLabel.Text = "No data to send. Type into the lower box and hit Enter to send.";
           return;
        }

        // Remove line return characters from string to send to echo server:
        String strToSend = strText.Substring(0, strText.Length - LINE_RETURN_SIZE);

        // Send the text line to the echo server and clear the send text box
        client.sendEchoText(String.Format("{0}", strToSend), statusLabel, receiveText);
 
        sendText.Text = "";
     }
  }   
} 